<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

if ( ! class_exists( 'Putler_Connector' ) ) {

	class Putler_Connector {

		private $email_address = '';
		private $api_token = '';
		private $version;
		private $batch_size = 100;
		public $text_domain;
		private $api_url;
		public $settings_url;

		protected static $instance = null;

		/**
		 * Call this method to get singleton
		 *
		 * @return Putler_Connector
		 */
		public static function getInstance() {
			if ( is_null( self::$instance ) ) {
				self::$instance = new self();
			}

			return self::$instance;
		}

		/**
		 * Putler_Connector Constructor.
		 *
		 * @access public
		 * @return void
		 */
		private function __construct() {

			$this->text_domain = 'putler_connector';
			$this->api_url     = 'https://web.putler.com/connectorAPI';

			$this->version = WPC_VERSION;

			$settings = get_option( 'putler_connector_settings', null );
			if ( ! empty( $settings ) ) {
				$api_token           = ( ! empty( $settings['api_token'] ) ) ? explode( ",", $settings['api_token'] ) : null;
				$this->email_address = ( ! empty( $settings['email_address'] ) ) ? $settings['email_address'] : null;

				if ( ! empty( $api_token ) ) {
					foreach ( $api_token as $token ) {
						if ( strpos( $token, 'web-' ) !== false ) {
							$this->api_token = $token;
							break;
						}
					}
				}
			}

			// Show a message when no web tokens found
			if ( empty( $this->api_token ) && ! empty( $settings ) ) {
				add_action( 'admin_notices', array( &$this, 'putler_desktop_deprecated' ) );
			}

			if ( is_admin() ) {
				$this->settings_url = admin_url( 'tools.php?page=putler_connector' );
				add_action( 'admin_menu', array( &$this, 'add_admin_menu_page' ) );
				add_action( 'wp_ajax_putler_connector_get_data', array( &$this, 'send_batch' ) );
				add_action( 'wp_ajax_putler_connector_get_temp_token', array( &$this, 'get_temp_token' ) );
				add_action( 'wp_ajax_putler_connector_get_auth_token', array( &$this, 'get_auth_token' ) );
				add_action( 'wp_ajax_putler_connector_connection_heartbeat', array( &$this, 'connection_heartbeat' ) );
				add_action( 'wp_ajax_putler_connector_resync', array( &$this, 'send_resync_request' ) );
				add_action( 'wp_ajax_putler_connector_sync_complete', array( &$this, 'resync_request_complete' ) );
			}

			add_action( 'wp_ajax_nopriv_putler_connector_get_data', array( &$this, 'send_batch' ) );
			add_action( 'wp_ajax_nopriv_putler_connector_get_temp_token', array( &$this, 'get_temp_token' ) );
			add_action( 'wp_ajax_nopriv_putler_connector_get_auth_token', array( &$this, 'get_auth_token' ) );
			add_action( 'wp_ajax_nopriv_putler_connector_sync_complete', array( &$this, 'resync_request_complete' ) );


		}

		public function putler_desktop_deprecated() {
			if ( empty( $this->api_token ) || empty( $this->email_address ) ) {
				echo '<div id="putler_configure_message" class="updated fade error"><p>' . sprintf( __( 'Putler Connector for Putler desktop has deprecated. Please upgrade to <strong><a href="%s" target="_blank">Putler Web</a></strong>.', $this->text_domain ), 'https://web.putler.com/' ) . '</p></div>';
			}
		}

		//function to generate temp pc token
		public function generate_random_string( $str_length, $type = 'Alphanumeric' ) {

			$str_alphanumeric = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
			$str_alphabet     = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
			$str_random       = '';

			switch ( $type ) {
				case 'Alphanumeric':
					{
						for ( $i = 0; $i < $str_length; $i ++ ) {
							$str_random .= $str_alphanumeric[ rand( 0, strlen( $str_alphanumeric ) - 1 ) ];
						}
						break;
					}
				case 'Alphabets':
					{
						for ( $i = 0; i < $str_length; $i ++ ) {
							$str_random .= $str_alphabet[ rand( 0, strlen( $str_alphabet ) - 1 ) ];
						}
						break;
					}
			}

			return $str_random;
		}

		public function add_admin_menu_page() {
			add_management_page( __( 'Putler Connector', $this->text_domain ), __( 'Putler Connector', $this->text_domain ), 'manage_options', 'putler_connector', array( &$this, 'display_page' ) );
		}

		public function display_page() {

			$authenticate = 0;

			$last_synced_date = get_option( 'sa_' . PUTLER_GATEWAY_PREFIX . '_last_updated' );
			$authenticated    = get_option( 'putler_connector_authenticated' );

			?>

            <script type="text/javascript">
                var send_resync_request = function () {
                    jQuery.ajax({
                        type: 'POST',
                        url: (ajaxurl.indexOf('?') !== -1) ? ajaxurl + '&action=putler_connector_resync' : ajaxurl + '?action=putler_connector_resync',
                        dataType: "text",
                        action: 'putler_connector_resync',
                        success: function (response) {
                            response = JSON.parse(response);
                            if (response.ack === 'Success') {
                                window.location.href = "<?php echo $this->settings_url;?>";
                            }
                        }
                    });
                };
            </script>

			<?php

			if ( empty( $authenticated ) ) {
				$authenticate = 1;

				if ( ( ! empty( $this->api_token ) && empty( $this->email_address ) ) || ( empty( $this->api_token ) && ! empty( $this->email_address ) ) ) { //code to delete both the email & token if only one of them is present.
					delete_option( 'putler_connector_settings' );
					$this->api_token = $this->email_address = '';
				}
			}

			$display_msg = '<span class="dashicons dashicons-yes" style="color:#0CCC0C;font-size: 2em;width: 1em;height: 1em;line-height: 0.7;"></span>' . __( "Putler is connected", $this->text_domain );

			if ( ! empty( $last_synced_date ) ) {
				$display_msg .= __( " & last sync date was ", $this->text_domain ) . $last_synced_date . '.';
			} else {
				$display_msg .= __( " & No orders have been synced yet.", $this->text_domain );
			}

			if ( ( ! empty( $_REQUEST['action'] ) && $_REQUEST['action'] == PUTLER_GATEWAY_PREFIX . '_activate' ) || ( empty( $this->api_token ) || empty( $this->email_address ) ) ) {
				$authenticate = 1;
			}

			if ( $authenticate == 1 ) {
				$display_msg = __( "Trying to Connect to Putler...", $this->text_domain );
			} else {

				$resync           = get_transient( 'putler_connector_resync' );
				$resync_on_update = get_option( '_' . PUTLER_GATEWAY_PREFIX . '_delete_and_resync' );

				if ( ! empty( $resync_on_update ) ) {
					$resync = 1;
					?>

                    <script type="text/javascript">
                        send_resync_request();
                    </script>

					<?php

					delete_option( '_' . PUTLER_GATEWAY_PREFIX . '_delete_and_resync' );
				}


				if ( ! empty( $resync ) ) {
					if ( ! empty( $_REQUEST['post_activation'] ) ) { //for handling display of message post activation
						$display_msg = __( "Your transactions are getting synced with Putler. Please check after some time.", $this->text_domain );
					} else {
						$display_msg = __( "Your resync request is under process and your transactions will be resynced soon.", $this->text_domain );
					}
				} else {
					$display_msg .= '</div> <div> <input type="submit" id="putler_connector_resync" class="button-primary" value="' . __( "Resync Data", $this->text_domain ) . '">';
				}

			}

			echo '<div class="wrap" id="putler_connector_settings_page" style="font-size: 1.1em;">
                    <h1>' . __( "Putler Connector", $this->text_domain ) . '</h1> <br/>
                    <div>' . $display_msg . '</div>
                  </div>';

			if ( $authenticate == 1 ) {
				$this->authenticate();
			}

			?>
            <script type="text/javascript">
                jQuery(document).on('ready', function () {
                    jQuery(document).on('click', '#putler_connector_resync', function () {
                        send_resync_request();
                    });
                });
            </script>


			<?php

		}

		public function authenticate() {

			$authenticate = 1;
			if ( ! empty( $this->api_token ) && ! empty( $this->email_address ) ) { //for existing users

				$result = $this->validate_api_info( $this->api_token, $this->email_address, 'validate', array( 'Site-URL' => site_url() ) );

				if ( ! is_wp_error( $result ) ) {

					$res_body = ( ! empty( $result['body'] ) ) ? json_decode( $result['body'], true ) : array();

					if ( ( ! empty( $result['response']['code'] ) && $result['response']['code'] == 200 ) &&
					     ( ! empty( $res_body['ack'] ) && $res_body['ack'] == "Success" ) ) {
						$authenticate = 0;

						update_option( 'putler_connector_authenticated', 1 );

						$msg = '<span>' . __( 'Successfully Authenticated!!!', $this->text_domain ) . ' </span> <span class="dashicons dashicons-yes" style="color:#0CCC0C;font-size: 2em;width: 1em;height: 1em;line-height: 0.7;"></span>';
						$this->show_message( $msg );

						?>
                        <script type="text/javascript">
                            setTimeout(function () {
                                window.location.href = "<?php echo $this->settings_url;?>";
                            }, 3000);
                        </script>
						<?php

					} else {
						delete_option( 'putler_connector_authenticated' );
						delete_option( 'putler_connector_settings' );

						?>

                        <script type="text/javascript">
                            setTimeout(function () {
                                window.location.href = "<?php echo $this->settings_url;?>";
                            }, 3000);
                        </script>

						<?php

						// $msg = __('Authentication Failed.', $this->text_domain).' <a href="">Try again</a>';
						// $this->show_message($msg);
						exit;
					}
				} else {
					delete_option( 'putler_connector_authenticated' );
					delete_option( 'putler_connector_settings' );

					?>

                    <script type="text/javascript">
                        setTimeout(function () {
                            window.location.href = "<?php echo $this->settings_url;?>";
                        }, 3000);
                    </script>

					<?php
					// $msg = __('Authentication Failed.', $this->text_domain).' <a href="">Try again</a>';
					// $this->show_message($msg);
					exit;
				}
			}

			if ( $authenticate == 1 ) { //for new users

				$existing_user = 1;

				if ( empty( $this->api_token ) ) {
					$this->api_token = $this->generate_random_string( 15 );
					update_option( 'putler_connector_temp_token', $this->api_token ); //TODO: retest & confirm
					$existing_user   = 0;
				}

				//getting temp token
				$result = $this->validate_api_info( $this->api_token, $this->email_address, 'get_temp_token', array( 'Site-URL' => site_url() ) );

				if ( ! is_wp_error( $result ) ) {

					$res_body = ( ! empty( $result['body'] ) ) ? json_decode( $result['body'], true ) : array();

					if ( ( ! empty( $result['response']['code'] ) && $result['response']['code'] == 200 ) &&
					     ( ! empty( $res_body['ack'] ) && $res_body['ack'] == "Success" ) ) {

						$msg = __( 'Authenticating...', $this->text_domain );
						$this->show_message( $msg );

						?>
                        <script type="text/javascript">
                            var start_timestamp = Date.now();
                            var pc_connection_heartbeat = function ( start_timestamp ) {

                                jQuery.ajax({
                                    type: 'POST',
                                    url: (ajaxurl.indexOf('?') !== -1) ? ajaxurl + '&action=putler_connector_connection_heartbeat' : ajaxurl + '?action=putler_connector_connection_heartbeat',
                                    dataType: "text",
                                    action: 'putler_connector_connection_heartbeat',
                                    success: function (response) {
                                        response = JSON.parse(response);
                                        if (response.ack == 'Success') {
                                            jQuery("#putler_connector_settings_page").append('<br/> <div><span>Successfully Authenticated!!!</span> <span class="dashicons dashicons-yes" style="color:#0CCC0C;font-size: 2em;width: 1em;height: 1em;line-height: 0.7;"></span></div>');

											<?php update_option( '_' . PUTLER_GATEWAY_PREFIX . '_delete_and_resync', 1, 'no' ); //for enabling resync on update ?>

                                            setTimeout(function () {
                                                window.location.href = "<?php echo $this->settings_url . '&post_activation=1';?>";
                                            }, 3000);

                                        } else {

                                        	current_timestamp = Date.now(); 
                                        	
                                        	if( current_timestamp - start_timestamp >= 30000 ) {
													jQuery("#putler_connector_settings_page").html('<h1><?php echo __( "Putler Connector", $this->text_domain ); ?></h1>'+
																									'<br/>'+
																									'<div style="background: lightyellow;border: 0.2em solid #c5c593;border-radius: 0.2em;padding: 0.75em 1em;">'+
							        																	'<h2>Something went wrong while authenticating!</h2>'+
							        																	'<div>Here\'s what to do next:'+
							            																	'<ul>'+
																											  '<li style="list-style-type:disc;margin-left:1em;margin-bottom:0.5em;">'+
																											  		'Do you have any security plugin active on your website?'+
																											  		'<br/>'+
																											  		'If you do, kindly go to your security plugin and whitelist this URL "<strong>https://web.putler.com/</strong>" Once whitelisted, try authenticating your store again.'+
																											  '</li>'+
																											  '<li style="list-style-type:disc;margin-left:1em;margin-bottom:0.5em;">If you still face the same issue even after white-listing OR you don\'t have any security plugin active - <a href="https://www.putler.com/contact-us/" target="_blank">send us a message</a>, we will get back to you ASAP</li>'+
																											'</ul>'+
							        																'</div>');
                                        	} else {
                                        		setTimeout(function ( start_timestamp ) {
	                                                pc_connection_heartbeat( start_timestamp );
	                                            }, 3000, start_timestamp);
                                        	}
                                        }
                                    }
                                });
                            }

                            pc_connection_heartbeat(start_timestamp);
                        </script>

						<?php

					} else {

						if ( empty( $existing_user ) ) {

							$msg = __( 'Authentication Failed.', $this->text_domain ) . '<br/> <br/> <div class="notice notice-error"> ' . __( "Please make sure that you have added an " . PUTLER_GATEWAY . " account in Putler. If you do not have a Putler account, you can create one for free and enjoy trial for 14 days.", $this->text_domain ) . ' <strong><i><a href="https://web.putler.com/#!/signup" target="_blank">' . __( 'Try Putler for free!', $this->text_domain ) . '</a></i></strong>. <br/> <br/>' . __( 'Once the ' . PUTLER_GATEWAY . ' account has been added successfully, please click ', $this->text_domain ) . '<a href="">' . __( 'here', $this->text_domain ) . '.</a> </div>';

						} else {
							$msg = __( 'Authentication Failed.', $this->text_domain ) . ' ' . __( 'You would need to reset the account in ', $this->text_domain ) . ' <strong><a href="https://web.putler.com/" target="_blank">' . __( 'Putler Web ', $this->text_domain ) . '</a></strong>';
						}

						$this->show_message( $msg );
					}
				} else {
					$msg = __( 'Authentication Failed.', $this->text_domain ) . ' <a href="">Try again</a>';
					$this->show_message( $msg );
				}
			}
		}

		public function connection_heartbeat() {

			$response = array( 'ack' => 'Failure' );

			$authenticated = get_option( 'putler_connector_authenticated' );
			$settings      = get_option( 'putler_connector_settings', null );

			if ( ! empty( $settings ) && ! empty( $authenticated ) ) {
				$this->api_token     = ( ! empty( $settings['api_token'] ) ) ? $settings['api_token'] : null;
				$this->email_address = ( ! empty( $settings['email_address'] ) ) ? $settings['email_address'] : null;

				if ( ! empty( $this->api_token ) && ! empty( $this->email_address ) ) {
					$response = array( 'ack' => 'Success' );
				}
			}

			ob_clean();
			echo json_encode( $response );
			die();
		}


		public function send_resync_request() {

			$response = array( 'ack' => 'Failure' );

			$result = $this->validate_api_info( $this->api_token, $this->email_address, 'delete_and_sync', array(
				'Gateway'  => PUTLER_GATEWAY,
				'Site-URL' => site_url()
			) );

			if ( ! is_wp_error( $result ) ) {
				set_transient( 'putler_connector_resync', 1, ( HOUR_IN_SECONDS * 3 ) );
				$response = array( 'ack' => 'Success' );
			}

			ob_clean();
			echo json_encode( $response );
			die();
		}


		public function resync_request_complete() {
			delete_transient( 'putler_connector_resync' );

			$response = array( 'ack' => 'Success' );
			ob_clean();
			echo json_encode( $response );
			die();
		}

		public function get_temp_token() {

			$this->api_token = get_option( 'putler_connector_temp_token', false );

			$temp_pc_token     = ( ! empty( $_SERVER['HTTP_X_PC_TEMP_TOKEN'] ) ) ? $_SERVER['HTTP_X_PC_TEMP_TOKEN'] : '';
			$temp_putler_token = ( ! empty( $_SERVER['HTTP_X_PUTLER_TEMP_TOKEN'] ) ) ? $_SERVER['HTTP_X_PUTLER_TEMP_TOKEN'] : '';

			if ( ! empty( $temp_pc_token ) && $temp_pc_token == $this->api_token ) {
				$response = array( 'ack' => 'Success', 'MESSAGE' => __( 'Authentication Successful', $this->text_domain ) );

				update_option( 'putler_connector_putler_temp_token', $temp_putler_token );

				$result = $this->validate_api_info( $this->api_token, $this->email_address, 'get_auth_token', array(
					'X-Putler-Temp-Token' => $temp_putler_token,
					'Site-URL'            => site_url()
				) );

				if ( ! is_wp_error( $result ) ) {

					$res_body = ( ! empty( $result['body'] ) ) ? json_decode( $result['body'], true ) : array();

					if ( ! ( ( ! empty( $result['response']['code'] ) && $result['response']['code'] == 200 ) &&
					         ( ! empty( $res_body['ack'] ) && $res_body['ack'] == "Success" ) ) ) {
						$msg = __( 'Authentication Failed.', $this->text_domain ) . ' <a href="">Try again</a>';
						$this->show_message( $msg );
					}
				} else {
					$msg = __( 'Authentication Failed.', $this->text_domain ) . ' <a href="">Try again</a>';
					$this->show_message( $msg );
				}

			} else {
				$response = array( 'ack' => 'Failure', 'MESSAGE' => __( 'Authentication Failure', $this->text_domain ) );
				$msg      = __( 'Authentication Failed.', $this->text_domain ) . ' <a href="">Try again</a>';
				$this->show_message( $msg );
			}

			ob_clean();
			echo json_encode( $response );
			die();
		}

		public function get_auth_token() {
			$this->api_token   = get_option( 'putler_connector_temp_token', false );
			$temp_putler_token = get_option( 'putler_connector_putler_temp_token', false );

			$temp_pc_token      = ( ! empty( $_SERVER['HTTP_X_PC_TEMP_TOKEN'] ) ) ? $_SERVER['HTTP_X_PC_TEMP_TOKEN'] : '';
			$temp_putler_token1 = ( ! empty( $_SERVER['HTTP_X_PUTLER_TEMP_TOKEN'] ) ) ? $_SERVER['HTTP_X_PUTLER_TEMP_TOKEN'] : '';

			if ( ( ! empty( $temp_pc_token ) && $temp_pc_token == $this->api_token ) && ( ! empty( $temp_putler_token1 ) && $temp_putler_token1 == $temp_putler_token ) ) {
				$response = array( 'ack' => 'Success', 'MESSAGE' => __( 'Authentication Successful', $this->text_domain ) );

				//save settings
				$settings = array();

				$this->email_address = $settings['email_address'] = ( ! empty( $_SERVER['HTTP_X_PUTLER_EMAIL'] ) ) ? $_SERVER['HTTP_X_PUTLER_EMAIL'] : '';
				$this->api_token     = $settings['api_token'] = ( ! empty( $_SERVER['HTTP_X_PUTLER_AUTH_TOKEN'] ) ) ? $_SERVER['HTTP_X_PUTLER_AUTH_TOKEN'] : '';

				$result = $this->validate_api_info( $this->api_token, $this->email_address, 'set_auth_token', array(
					'X-Putler-Temp-Token' => $temp_putler_token,
					'Gateway'             => PUTLER_GATEWAY,
					'AccountName'         => get_bloginfo( 'name' ),
					'Site-URL'            => site_url()
				) );
				if ( ! is_wp_error( $result ) ) {

					$res_body = ( ! empty( $result['body'] ) ) ? json_decode( $result['body'], true ) : array();

					if ( ( ! empty( $result['response']['code'] ) && $result['response']['code'] == 200 ) &&
					     ( ! empty( $res_body['ack'] ) && $res_body['ack'] == "Success" ) ) {

						delete_option( 'putler_connector_temp_token' );
						delete_option( 'putler_connector_putler_temp_token' );

						update_option( 'putler_connector_settings', $settings );
						update_option( 'putler_connector_authenticated', 1 );

						?>
                        <script type="text/javascript">
                            jQuery("#putler_connector_settings_page").append('<br/> <br/> <div></div>');
                        </script>
						<?php

					} else {
						$msg = __( 'Authentication Failed.', $this->text_domain ) . ' <a href="">Try again</a>';
						$this->show_message( $msg );
					}
				} else {
					$msg = __( 'Authentication Failed.', $this->text_domain ) . ' <a href="">Try again</a>';
					$this->show_message( $msg );
				}

			} else {
				$response = array( 'ack' => 'Failure', 'MESSAGE' => __( 'Authentication Failure', $this->text_domain ) );
				$msg      = __( 'Authentication Failed.', $this->text_domain ) . ' <a href="">Try again</a>';
				$this->show_message( $msg );
			}

			ob_clean();
			echo json_encode( $response );
			die();
		}

		//Function to display authentication messages
		private function show_message( $message ) {
			if ( ! empty( $message ) ) {
				?>
                <script type="text/javascript">
                    var message = '<?php echo $message; ?>';
                    jQuery("#putler_connector_settings_page").append('<br/> <div>' + message + '</div>');
                </script>
				<?php
			}
		}

		private function validate_api_info( $token, $email, $action = '', $headers = array() ) {
			// Validate with API server
			$result = wp_remote_post( $this->api_url,
				array(
					'headers' => array_merge( array(
						'Authorization' => 'Basic ' . base64_encode( $email . ':' . $token ),
						'User-Agent'    => 'Putler Connector/' . $this->version,
					), $headers ),
					'body'    => array( 'action' => $action )
				)
			);

			return $result;
		}

		public function generate_xml_from_array( $array, $node_name ) {
			$xml = '';
			if ( is_array( $array ) || is_object( $array ) ) {
				foreach ( $array as $key => $value ) {
					if ( is_numeric( $key ) ) {
						$key = $node_name;
					}

					$node_value = '';

					if ( ! empty( $value ) ) {
						$node_value = "\n" . $this->generate_xml_from_array( $value, $node_name );
					}

					$xml .= '<' . $key . '>' . $node_value . '</' . $key . '>' . "\n";
				}
			} else {
				$xml = htmlspecialchars( $array, ENT_QUOTES ) . "\n";
			}

			return $xml;
		}


		public function generate_valid_xml_from_array( $array, $node_block = PUTLER_GATEWAY, $node_name = 'node' ) {
			$xml = '<?xml version="1.0" encoding="UTF-8" ?>' . "\n";
			$xml .= '<!--email_off--><' . $node_block . '>' . "\n";
			$xml .= $this->generate_xml_from_array( $array, $node_name );
			$xml .= '</' . $node_block . '><!--email_off-->' . "\n";

			return $xml;
		}

		public function send_batch() {
			$response = array( 'status' => 'OK', 'message' => '', 'results' => array() );

			$is_valid = 0;

			if ( ! empty( $_REQUEST['AUTH'] ) ) {
				$auth = base64_decode( $_REQUEST['AUTH'] );
				if ( ! empty( $auth ) ) {
					$credentials = explode( ":", $auth );
				} else {
					$credentials = array();
				}
				$email = ( ! empty( $credentials[0] ) ) ? $credentials[0] : '';
				$token = ( ! empty( $credentials[1] ) ) ? $credentials[1] : '';

				if ( $email != $this->email_address || $token != $this->api_token ) {
					$is_valid = 1;
				}

			} else {
				$is_valid = 1;
			}

			header( 'Content-Type: text/xml' );
			while ( ob_get_contents() ) {
				ob_clean();
			}

			if ( ! empty( $is_valid ) ) {
				$response = array( 'ACK' => 'Failure', 'MESSAGE' => __( 'Authentication Failure', $this->text_domain ) );
				$response = $this->generate_valid_xml_from_array( $response, PUTLER_GATEWAY );
				die( $response );
			}

			if ( empty( $_REQUEST['STARTDATE'] ) || empty( $_REQUEST['ENDDATE'] ) ) {
				$response = array( 'ACK' => 'Failure', 'MESSAGE' => __( 'Params Missing', $this->text_domain ) );
				$response = $this->generate_valid_xml_from_array( $response, PUTLER_GATEWAY );
				die( $response );
			}
			update_option( 'sa_' . PUTLER_GATEWAY_PREFIX . '_last_updated', current_time( 'Y-m-d H:i:s' ) ); //updating the last synced time

			$offset       = ( ! empty( $_REQUEST['OFFSET'] ) ) ? $_REQUEST['OFFSET'] : 0;
			$sub_offset   = ( ! empty( $_REQUEST['SUBOFFSET'] ) ) ? $_REQUEST['SUBOFFSET'] : 0;
			$limit        = ( ! empty( $_REQUEST['LIMIT'] ) ) ? $_REQUEST['LIMIT'] : 100;
			$send_headers = ( isset( $_REQUEST['SENDHEADERS'] ) ) ? $_REQUEST['SENDHEADERS'] : 'true';

			//Getting the data from ecommerce plugins
			$params = array( 'start_date' => $_REQUEST['STARTDATE'], 'end_date' => $_REQUEST['ENDDATE'], 'offset' => $offset, 'sub_offset' => $sub_offset, 'limit' => $limit );
			$params = apply_filters( 'putler_connector_get_orders', $params );

			// Check if all orders are received...
			foreach ( (array) $params as $connector => $orders ) {

				$ocount           = ( ! empty( $orders['count'] ) ) ? $orders['count'] : 0;
				$prev_start_limit = ( ! empty( $orders['last_start_limit'] ) ) ? $orders['last_start_limit'] : 0;

				// Send one batch to the server
				if ( ! empty( $orders['data'] ) && is_array( $orders['data'] ) && count( $orders['data'] ) > 0 ) {

					if ( $connector == PUTLER_GATEWAY ) { //New API Code
						$data = ( ! empty( $orders['data'] ) && is_array( $orders['data'] ) && count( $orders['data'] ) > 0 ) ? $this->array_to_csv( $orders['data'], $send_headers ) : '';

						if ( $ocount < $limit ) {
							$ack    = 'Success';
							$offset = 0;
						} else {
							$ack    = 'SuccessWithWarning';
							$offset = $ocount + $prev_start_limit;
						}

						$response = array( 'ACK' => $ack, 'DATA' => $data, 'OFFSET' => $offset );

						update_option( 'sa_' . PUTLER_GATEWAY_PREFIX . '_last_updated', current_time( 'Y-m-d H:i:s' ) ); //updating the last synced time

						$response = $this->generate_valid_xml_from_array( $response, PUTLER_GATEWAY );
					}
				} else {
					$response = array( 'ACK' => 'Success', 'DATA' => '', 'OFFSET' => ( $ocount + $prev_start_limit ) );
					$response = $this->generate_valid_xml_from_array( $response, PUTLER_GATEWAY );
				}
			}

			die( $response );
		}

		public function array_to_csv( $orders, $send_headers = 'true' ) {
			if ( empty( $orders ) ) {
				return '';
			}

			$oid = ob_start();
			$fp  = fopen( 'php://output', 'a+' );
			foreach ( (array) $orders as $index => $row ) {
				if ( $index == 0 && $send_headers == 'true' ) {
					fputcsv( $fp, array_keys( $row ) );
				}
				fputcsv( $fp, $row );
			}
			fclose( $fp );
			$csv_data = ob_get_clean();
			if ( ob_get_clean() > 0 ) {
				ob_end_clean();
			}

			return $csv_data;
		}

	}
}

